/* Jatha -- bringing to Java some of the power of lisp.
 * Copyright (C) 2001  Kimberley Burchett   kimbly@pobox.com
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package macros;

import java.io.*;
import jatha.*;

/**
 * <code>CTOR(prop1, prop2, ...)</code>
 * 
 * <p>The CTOR (short for constructor) macro declares a constructor.
 * In order to use the CTOR macro, you must have used the CLASS macro
 * when declaring your class, and the PROP macro when declaring the
 * properties.  Example:
 * 
 * <pre>
 * public @CLASS(Point) {
 *     @PROP(int, x);
 *     @PROP(int, y);
 *     public @CTOR(x, y);
 * }
 * </pre>
 *
 * Expands into this:
 *
 * <pre>
 * public class Point {
 *     private int m_x;
 *     public int getX() { return m_x; }
 *     public void setX(int x) { m_x = x; }
 * 
 *     private int m_y;
 *     public int getY() { return m_y; }
 *     public void setY(int y) { m_y = y; }
 * 
 *     public Point(int x, int y) {
 *         m_x = x;
 *         m_y = y;
 *     }
 * }
 * </pre>
 *
 * <p>Note: no class modifiers are generated, so by default the constructor 
 * only has package visibility.  If you want to make the constructor public 
 * or private, you need to put modifiers before the macro.  For example:
 * <code>public&nbsp;@CTOR(x,&nbsp;y)</code>.
 * 
 * @see macros.CLASS
 * @see macros.PROP
 */
public class CTOR extends Macro {
	/** Expands the macro */
	public void expand(String[] args, Writer out, Expander expander) throws IOException {
		String className = (String)expander.getMessage("CLASS`name");
		if (className == null)
			expander.error("@CTOR macro requires that you used the @CLASS macro when declaring your class");

		String[] types = new String[args.length];
		for (int i=0; i < args.length; ++i) {
			types[i] = (String)expander.getMessage("PROP`typeof`"+args[i]);
			if (types[i] == null) {
				expander.error("Looks like you didn't use the @PROP macro when declaring property \""+args[i]+"\"");
			}
		}

		out.write(className+"(");
		for (int i=0; i < args.length; ++i) {
			if (i != 0)
				out.write(", ");
			out.write(types[i]+" ");
			out.write(PROP.getNameOfParameter(types[i], args[i]));
		}
		out.write(") {\n");
		for (int i=0; i < args.length; ++i) {
			String member = PROP.getNameOfMemberVariable(args[i]);
			String param = PROP.getNameOfParameter(types[i], args[i]);
			out.write("\t\t"+member+"="+param+";\n");
		}
		out.write("\t}\n");
	}
}
