/* Jatha -- bringing to Java some of the power of lisp.
 * Copyright (C) 2001  Kimberley Burchett   kimbly@pobox.com
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package macros;

import java.io.*;
import jatha.*;

/**
 * <code>ENUM_OBJ(className, elem1, name1, elem2, name2, ...)</code>
 * 
 * <p>This macro creates an object-based enumeration.  That is, each element 
 * of the enumeration is represented as a separate object (as opposed to a
 * C-style enumeration which represent elements as integers).  Object-based
 * enumerations are type-safe, while integer-based ones are not.
 * 
 * Example: <code>@ENUM_OBJ(Type, INT, "int", DOUBLE, "double")</code>
 * will generate something like this:
 * <pre>
 * class Type {
 *     public static final Type INT = new Type("int");
 *     public static final Type DOUBLE = new Type("double");
 * 
 *     public static int getNumElements() {
 *         return 2;
 *     }
 *     public static Type getElement(int __index) {
 *         switch (__index) {
 *         case 0: return INT;
 *         case 1: return DOUBLE;
 *         default: return null;
 *         }
 *     }
 *     public static Type getElement(String __name) {
 *         if (__name.equals("int")) return INT;
 *         if (__name.equals("double")) return DOUBLE;
 *         return null;
 *     }
 *     private Type(String __description) {
 *         this.__description = __description;
 *     }
 *     private String __description;
 *     public String toString() {
 *         return __description;
 *     }
 * }
 * </pre>
 * 
 * <p>Note: this macro declares an entire class, unlike ENUM_INT and ENUM_BITS
 * which only declare some member variables and methods.
 * 
 * <p>Also note: no class modifiers are generated, so by default the class 
 * only has package visibility.  If you want to make the class public or 
 * private, you need to put modifiers before the macro.  For example:
 * <code>public @ENUM_OBJ(...)</code>.
 * 
 * @see macros.ENUM_INT
 * @see macros.ENUM_BITS
 */
public class ENUM_OBJ extends Macro {
	/** Expands the macro */
	public void expand(String[] args, Writer out, Expander expander) throws IOException {
		if (args.length < 3)
			expander.error("ENUM_OBJ requires at least three arguments");
		if ((args.length % 2) != 1)
			expander.error("ENUM_OBJ must have an odd number of arguments");
		String type = args[0];
		int numElements = args.length / 2;

		out.write("class "+type+" {\n");
		for (int i=1; i < args.length; i+=2) {
			String varName = args[i];
			String strName = args[i+1];
			out.write("\tpublic static final "+type+" "+varName+
				" = new "+type+"("+strName+");\n");
		}
		out.write("\n");

		out.write("\tpublic static int getNumElements() {\n");
		out.write("\t\treturn "+numElements+";\n");
		out.write("\t}\n");

		out.write("\tpublic static "+type+" getElement(int __index) {\n");
		out.write("\t\tswitch (__index) {\n");
		for (int i=1; i < args.length; i+=2) {
			String varName = args[i];
			out.write("\t\tcase "+i/2+": return "+varName+";\n");
		}
		out.write("\t\tdefault: return null;\n");
		out.write("\t\t}\n");
		out.write("\t}\n");

		out.write("\tpublic static "+type+" getElement(String __name) {\n");
		for (int i=1; i < args.length; i+=2) {
			String varName = args[i];
			String strName = args[i+1];
			out.write("\t\tif (__name.equals("+strName+")) return "+varName+";\n");
		}
		out.write("\t\treturn null;\n");
		out.write("\t}\n");

		out.write("\tprivate "+type+"(String __description) {\n");
		out.write("\t\tthis.__description = __description;\n");
		out.write("\t}\n");

		out.write("\tprivate String __description;\n");
		
		out.write("\tpublic String toString() {\n");
		out.write("\t\treturn __description;\n");
		out.write("\t}\n");

		out.write("}\n");
	}
}
