/* Jatha -- bringing to Java some of the power of lisp.
 * Copyright (C) 2001  Kimberley Burchett   kimbly@pobox.com
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package macros;

import java.io.*;
import jatha.*;

/**
 * <code>STRINGIFY(expr)</code>
 * 
 * <p>This macro makes dynamically generated strings a little easier to read.  
 * By default, the entire argument to STRINGIFY is wrapped in quotes.  However, 
 * if a backquote (`) is found, the runtime value of the backquoted expression 
 * is inserted into the string.  You can either backquote an identifier or an 
 * expression enclosed in parentheses.
 *
 * <p>Example:
 * <br><code>@STRINGIFY(x == `x; a(b)+c == `(a(b)+c);)</code> is equivalent to
 * <br><code>"x == "+x+"; a(b)+c == "+(a(b)+c)+";"</code>
 */
public class STRINGIFY extends Macro {
	/** Expands the macro */
	public void expand(String arg, Writer out, Expander expander) throws IOException {
		int length = arg.length();
		int index = 0;
		while (index < length) {
			// process any plain string up to the first backquote
			int backquoteIndex = arg.indexOf('`', index);
			if (backquoteIndex == -1) {
				String str = arg.substring(index);
				out.write('"'+util.escapify(str)+'"');
				return;
			}
			else if (backquoteIndex != index) {
				String str = arg.substring(index, backquoteIndex);
				// Note the + at the end -- we know more is coming
				out.write('"'+util.escapify(str)+"\"+");
			}

			// is it a complex expression?
			if (arg.charAt(backquoteIndex+1) == '(') {
				int stopIndex = util.findMatchingEndParen(arg, backquoteIndex+1);
				if (stopIndex == -1)
					expander.error("unmatched open parenthesis in backquote");
				// spit out the whole expression, including the parens
				// Note: here stopIndex is the ')', and we want to print it.
				out.write(arg.substring(backquoteIndex+1, stopIndex+1));
				index = stopIndex+1;	// continue after the expression
			}
			else {	// it's just a simple identifier
				int stopIndex;
				for (stopIndex=backquoteIndex+1; stopIndex < length; ++stopIndex)
					if (!Character.isJavaIdentifierPart(arg.charAt(stopIndex)))
						break;
				// Note: here stopIndex is a non-identifier character
				// or the end of the string -- either way, we don't want
				// to print it.
				out.write(arg.substring(backquoteIndex+1, stopIndex));
				index = stopIndex;	// continue after the identifier
			}
			// if there's any string left, print a +
			if (index < length)
				out.write("+");
		}
	}
}
